
;;;list of names that confuse the regexes, so get renamed by hand
;;; include anything that uses 's' or 'sv' type specifier, or that
;;; ends in 'd' that isn't intended to mean 'double'

(defparameter *special-case-list*
  '(("Rects" . "rect-s")
    ("Rectsv" . "rect-sv")
    ("Indexs" . "index-s")
    ("GlobalAlphaFactorsSUN" . "global-alpha-factor-s-sun")    
    ("TexSubImage4DSGIS" . "tex-sub-image-4d-sgis")
    ("TexImage4DSGIS" . "tex-image-4d-sgis")
    ;;("ShaderOp1EXT" . "shader-op-1-ext")
    ;;("ShaderOp2EXT" . "shader-op-2-ext")
    ;;("ShaderOp3EXT" . "shader-op-3-ext")

    ("UniformMatrix2x3fv" . "uniform-matrix-2x3-fv")
    ("UniformMatrix3x2fv" . "uniform-matrix-3x2-fv")
    ("UniformMatrix2x4fv" . "uniform-matrix-2x4-fv")
    ("UniformMatrix4x2fv" . "uniform-matrix-4x2-fv")
    ("UniformMatrix3x4fv" . "uniform-matrix-3x4-fv")
    ("UniformMatrix4x3fv" . "uniform-matrix-4x3-fv")
    ("GetQueryObjecti64vEXT" . "get-query-object-i64v-ext")
    ("GetQueryObjectui64vEXT" . "get-query-object-ui64v-ext")
))


(defparameter *whole-words* '("push" "depth" "mesh" "finish" "flush"
                              "attach" "detach" "through" "width"
                              "interleaved" "load" "end" "bind" "named"
                              "grid" "coord" "read" "blend" "compressed"
                              "attached" "enabled"
                              "attrib"
                              "multi"
                              "status"
                              "mapped"
                              ;?"env"
                              ))
;;; words ending in 's'...
;;; is arrays textures names pixels lists attribs parameters programs
;;; queries fences points maps pos buffers elements renderbuffers 
;;; framebuffers status shaders symbols bounds pntriangles
(defun fix-type-suffix (name)
  (cond
    ((string-equal name "nv") name) ; vendor name, not vector
    ((cl-ppcre:scan "^[0-9]N?u?[hbifds]v?$" name) name) ;count+type
    ((member name *whole-words* :test #'string-equal) name)
    ;;fix [1-3]D(EXT|ARB) (TexImage1DEXT etc) (but don't break -3dfx)
    ((cl-ppcre:scan "[1-3]D(EXT|ARB)" name)
     ;;fixme: stop running the regex twice here...
     (cl-ppcre:regex-replace "([1-3]D)(EXT|ARB)" name "\\1-\\2")
     )
    ;;fix Op1ATI etc from ATI_fragment_shader and Op1EXT from ext_vertex_shader
    ((cl-ppcre:scan "[1-3](ATI|EXT)" name)
     (cl-ppcre:regex-replace "([1-3])(EXT|ATI)" name "\\1-\\2"))
    (t ; anything else, try to split
     (cl-ppcre:regex-replace-all 
      ;;#
      ;;N (normalized VertexAttrib)
      ;;b s i f d ub us ui
      ;;v
      ;; (can't catch the u?i64v? types here, so special case them for now...)
      ;; 's' catches too many plurals, so skipping for now...
      ;; (just rect,index anyway, can special case them...)
      "([a-z][a-tv-z])((?:h|b|i|f|d|ub|us|ui|hv|bv|iv|fv|dv|sv|ubv|usv|uiv|v)|[0-9]+)$" name
      #'(lambda (match r1 r2)
          (declare (ignore match))
          (format nil "~a-~a" r1 r2))
      :simple-calls t
      )
)))
(defun fix-type-suffixes (name)
  (progn ;fix-special-cases
    (cl-ppcre:regex-replace-all 
     "(^|-)([^-]+)(-|$)" name
     #'(lambda (match r1 r2 r3)
         (declare (ignore match))
         (format nil "~a~a~a" (or r1 "") (fix-type-suffix r2) (or r3 "")))
     :simple-calls t
     )))

(defun mixedcaps->lcdash (str)
  (string-downcase
   (if (assoc str *special-case-list* :test #'string=)
       (cdr (assoc str *special-case-list* :test #'string=))
       (fix-type-suffixes (cl-ppcre:regex-replace-all 
                           "([a-z])([0-9A-Z])" str 
                           #'(lambda (match r1 r2)
                               (declare (ignore match))
                               (format nil "~a-~a" r1 r2))
                           :simple-calls t
                           )))))

(defun fix-gl-function-name (name)
  (mixedcaps->lcdash name))

